program X8255LOOP;
uses dos;

{$I hexread}
{$I hprint}
{$I beepy}

const
AllIn  : byte = $9B;
AllOut : byte = $80;

var
BasePort1 : word;
BasePort2 : word;

procedure BadBeeps;
begin
  BeepOn(800);
  BeepDelay(100);
  BeepOn(1500);
  BeepDelay(50);
  BeepOff;
end;

procedure HappyBeeps;
begin
  BeepOn(1000);
  BeepDelay(24);
  BeepOn(1200);
  BeepDelay(12);
  BeepOn(1000);
  BeepDelay(24);
  BeepOn(1200);
  BeepDelay(12);
  BeepOn(1000);
  BeepDelay(24);
  BeepOn(1200);
  BeepDelay(12);
  BeepOff;
end;

procedure SetPPMode(baseport: word; themode : byte);
var
conport : word;
begin
  conport := baseport or 3;
  port[conport] := themode;
end;

procedure OutOneZero(baseport : word; thebit : word);
var
dbyte : byte;
begin                  { set one bit in the 24 bit output field of our 82C55 low }
                       { baseport is the base port of the 82C55 }
  port[baseport]   := $FF;
  port[baseport+1] := $FF;
  port[baseport+2] := $FF;
  baseport := baseport or (thebit div 8);
  dbyte := 1;
  dbyte := dbyte shl (thebit mod 8);
  dbyte := dbyte xor $FF;
  port[baseport] := dbyte
end;

procedure OutOneOne(baseport : word; thebit : word);
var
dbyte : byte;
bitport : word;
begin                  { set one bit in the 24 bit output field of our 82C55 high }
                       { baseport is the base port of the 82C55 }
  port[baseport]   := $00;
  port[baseport+1] := $00;
  port[baseport+2] := $00;
  bitport := baseport or (thebit div 8);
  dbyte := 1;
  dbyte := dbyte shl (thebit mod 8);
  port[bitport] := dbyte
end;

function CheckOneZero(baseport : word; thebit : byte): boolean;
var
dbyte : byte;
tbyte : byte;
pbyte : byte;
bitport : word;
checkport : word;
portok : boolean;
begin                  { check for one low bit only in the 24 bit output field of our 82C55 }
                       { baseport is the base port of the 82C55 }
  portok := true;
  bitport := baseport or (thebit div 8);
  dbyte := 1;
  dbyte := dbyte shl (thebit mod 8);
  dbyte := dbyte xor $FF;
  for checkport := baseport to (baseport + 2) do
  begin
    if checkport = bitport then tbyte := dbyte else tbyte := $FF;
    pbyte := port[checkport];
    if pbyte <> tbyte then
    begin
      portok := false;
      write('Error during rotating zero test, expected ');
      HexPrint(tbyte,2);
      write(' but found ');
      HexPrint(pbyte,2);
      write(' at port ');
      HexPrint(CheckPort,4);
      writeln;
    end;
  end;
  CheckOneZero := portok;
end;

function CheckOneOne(baseport : word; thebit : byte): boolean;
var
dbyte : byte;
tbyte : byte;
pbyte : byte;
bitport : word;
checkport : word;
portok : boolean;
begin                  { check for one high bit only in the 24 bit output field of our 82C55 }
                       { baseport is the base port of the 82C55 }
  portok := true;
  bitport := baseport or (thebit div 8);
  dbyte := 1;
  dbyte := dbyte shl (thebit mod 8);
  for checkport := baseport to (baseport + 2) do
  begin
    if checkport = bitport then tbyte := dbyte else tbyte := $00;
    pbyte := port[checkport];
    if pbyte <> tbyte then
    begin
      portok := false;
      write('Error during rotating one test, expected ');
      HexPrint(tbyte,2);
      write(' but found ');
      HexPrint(pbyte,2);
      write(' at port ');
      HexPrint(CheckPort,4);
      writeln;
    end;
  end;
  CheckOneOne := portok;
end;

procedure BumOut(s: string);
begin
  BadBeeps;
  writeln(s);
  halt(2);
end;

procedure DoRotBitTest(outport,inport : word);
var
testbit : word;
times : word;

begin

  write('Rotating bit test, output port = ');
  HexPrint(outport,4);
  write(' and input port = ');
  HexPrint(inport,4);
  writeln;
  SetPPMode(outport,AllOut);
  SetPPMode(inport,AllIn);
  for times := 1 to 100 do
  begin
    for testbit := 0 to 23 do
    begin
      OutOneOne(outport,testbit);
      if not CheckOneOne(inport,testbit) then BumOut('PPORT LOOPBACK FAILURE!');
      OutOneZero(outport,testbit);
      if not CheckOneZero(inport,testbit) then BumOut('PPORT LOOPBACK FAILURE!');
    end;
  end;
end;

procedure TestPorts;
begin
  DoRotBitTest(BasePort1,BasePort2);
  DoRotBitTest(BasePort2,BasePort1);
end;

procedure PrintUsage;
begin
  writeln;
  writeln('8255LOOP -- Test 8255 type parallel ports via loopback cable');
  writeln;
  writeln('USAGE -- 8255LOOP PORT1 PORT2');
  writeln;
  writeln('Where PORT1 and PORT2 are hex 8255 base port addresses');
  writeln('Requires 2 uncommitted 8255s with all I/O from one chip');
  writeln('looped back to the other chip with an external cable');
  halt(2);
end;

procedure GetPorts;

begin
  if (ParamCount <> 2) then PrintUsage;
  if not HexWordRead(ParamStr(1),BasePort1) then BumOut('Undecipherable hex number > '+ParamStr(1));
  if not HexWordRead(ParamStr(2),BasePort2) then BumOut('Undecipherable hex number > '+ParamStr(2));;
  if BasePort1 = BasePort2 then BumOut('Ports 1 and 2 must be on different chips!');
  if (BasePort1 and 3) <> 0 then BumOut('Port 1 address not valid 8255 base port!');
  if (BasePort2 and 3) <> 0 then BumOut('Port 2 address not valid 8255 base port!');
end;

begin
  GetPorts;
  TestPorts;
  HappyBeeps;
  Writeln('PPORT TESTS OK!');
end.